#!/usr/bin/perl

######################################################################
package Cognition::uF::hAtom;
######################################################################

use XML::LibXML qw(:all);
use strict;
use CGI::Util;
use Digest::SHA1 qw(sha1_hex);
use Cognition::uF;
use Cognition::uF::hEntry;

BEGIN
{
	our $VERSION = 0.1;
	our @ISA = qw(Cognition::uF);
}


sub consume
{
	my $page   = shift;

	# hAtom	
	if ( $page->{Profiles}->{'http://purl.org/uF/hAtom/0.1/'}
	||  !$page->{options}->{uf_strict_profiles})
	{
		$page->{uF}->{hAtom} = [ Cognition::uF::hAtom::parse_all($page) ];
		foreach my $hat (@{ $page->{uF}->{hAtom} })
		{
			$page->mergeSubject( $hat->rdf_subject );
	
			foreach my $h ( @{ $hat->{entry} } )
			{
				$page->mergeSubject( $h->rdf_subject );
			}
		}
		if (defined $page->{uF}->{hAtom}->[0])
			{ $page->mergeNS( $page->{uF}->{hAtom}->[0] ); }		
	}
}


sub parse_all
{
	my $page   = shift;
	my $within = shift || $page->{DOM};
	my $fullPageMode;
	my @nodes  = Cognition::HTMLParser::Utils::searchClass('hfeed', $within);
	my @rv;
	
	if (!@nodes && $within->nodeType==XML_DOCUMENT_NODE)
	{
		push @nodes, $within->getDocumentElement ;
		$fullPageMode = 1;
	}
	elsif (!@nodes)
	{
		push @nodes, $within;
	}
	
	foreach my $hatom (@nodes)
	{
		my $hatom_parsed = parse($page, $hatom);
		push @rv, $hatom_parsed; 
	}
	
	$rv[0]->{_id} = $page->abs_url if ($fullPageMode);
	
	return @rv;
} #/sub parse_all


sub uri
{
	my $this = shift;
	my $all  = shift;
	my @rv;
	
	if ($this->{_dom}->nodeType == XML_DOCUMENT_NODE || $this->{_dom}->tagName eq 'html')
	{
		push @rv, $this->{_page}->abs_url;
	}
	elsif (length $this->{_id})
	{
		push @rv, $this->{_page}->abs_url.'#'.$this->{_id};
	}
	push @rv, Cognition::HTMLParser::Utils::BNodeURI($this->{_dom}, 'Feed')
		unless (@rv);
	#push @rv, Cognition::HTMLParser::Utils::XPointer($this->{_dom}, $this->{_page}->abs_url);
	
	return $rv[0] unless (defined $all);
	while ($all) { shift @rv; $all--; }
	return @rv;
} #/sub uri


sub rdf_subject
{
	my $this    = shift;
	my $ns      = 'urn:ietf:rfc:4287#';

	# Create RDF subject
	my $subject = Cognition::RDFModel::Subject->new($this->uri);
	$subject->addObject($this);
	$subject->setDom($this->{_dom});
	
	return $subject unless ($this->{entry});
	
	$subject->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#type',
		Cognition::RDFModel::Value->new($ns.'feed', 1));

	# Patterns for simple data.
	my @_OBJECTS  = qw(entry category);
	
	# Embedded objects which may be represented as resources
	foreach my $property (@_OBJECTS)
	{
		foreach my $V (@{ $this->{$property} })
		{
			my $val;
			if (ref $V)
				{ $val = Cognition::RDFModel::Value->new($V->uri, 1); }
			else
				{ $val = Cognition::RDFModel::Value->new($V); }
			$subject->addVal($ns.$property, $val);
		}
	}
	
	return $subject;
} #/sub rdf_subject


sub metadata_ns
{
	my $this = shift;
	# We use an IETF URN namespace because although we reuse the semantics of
	# http://www.w3.org/2005/Atom, we can't guarantee that output from this
	# module will conform to the Atom spec (e.g. content model for various
	# elements).
	return { 'atom' => { nsurl=>'urn:ietf:rfc:4287#',title=>'Atom' } };
} #/sub metadata_ns


sub parse
{
	my $page = shift;
	my $rv = { '_dom'=>shift };
	my $root_node = $rv->{'_dom'}->cloneNode(1);

	my $id = $root_node->getAttribute('id');
	$rv->{_id} = $id if (length $id);
	$rv->{_page} = $page;
	
	# Expand microformat patterns.
	my $incl_iterations = 0;
	my $replacements = 1;
	while (($incl_iterations < Cognition::uF::MAX_INCLUDE_ITERATIONS) && $replacements)
	{
		$replacements = Cognition::uF::ufp_include($root_node, $page) 
		              + Cognition::uF::ufp_include2($root_node, $page);
		$incl_iterations++;
	}
	Cognition::uF::ufp_abbr($root_node, $page);
	Cognition::uF::ufp_table($root_node, $page);
	
	# Pull out nested hEntrys
	my @E = Cognition::uF::hEntry::parse_all($page, $root_node);
	$rv->{entry} = \@E unless (!@E);

	# If no entries, then we're not really a feed, so don't both with the other
	# stuff.
	if ($rv->{entry})
	{
		# Now that we have reached here, no other composite microformats have
		# any business being nested within this root element. So in the interest
		# of Microformat opacity, let's destroy them.
		Cognition::uF::destroyer($root_node, 'vcard');  # Save hCards!
		
		# Author
		my @nested = Cognition::HTMLParser::Utils::searchClass('vcard', $root_node);
		foreach my $h (@nested)
		{
			if ($h->getAttribute('class') =~ / (^|\s) author (\s|$) /x)
			{
				push @{ $rv->{author} }, Cognition::uF::hCard::parse($page, $h);
			}
		}	
		if (!defined $rv->{author})
		{
			foreach my $h (@nested)
			{
				if ($h->tagName == 'address')
				{
					push @{ $rv->{author} }, Cognition::uF::hCard::parse($page, $h);
				}
			}
			if (!defined $rv->{author})
			{
				foreach my $address ($root_node->getElementsByTagName('address'))
				{
					push @{ $rv->{author} }, Cognition::uF::hCard::parse_all($page, $address);
				}
			}
		}
		
		# Categories
		my @tags = Cognition::uF::RelTag::parse_all($page, $root_node);
		$rv->{category} = \@tags unless (!@tags);	
		
		# Null entry authors should be set to feed author.
		foreach my $e (@{$rv->{entry}})
		{
			$e->{author} = $rv->{author} unless ($e->{author});
		}
		
	}
	
	bless $rv;
	return $rv;
} #/sub parse


1; #/package Cognition::uF::hAtom




